@@ -1,7 +1,5 @@
 /* printif.c -- print an interface configuration
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021
-  Free Software Foundation, Inc.
+  Copyright (C) 2001-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -48,7 +46,7 @@
 #include <arpa/inet.h>
 #include "ifconfig.h"
 #include "xalloc.h"
-#include <unused-parameter.h>
+#include <attribute.h>
 
 FILE *ostream;			/* Either stdout or stderror.  */
 int column_stdout;		/* The column position of the cursor on stdout.  */
@@ -120,7 +118,7 @@ struct format_handle format_handles[] = {
 /* Various helper functions to get the job done.  */
 
 void
-put_char (format_data_t form _GL_UNUSED_PARAMETER, char c)
+put_char (format_data_t form MAYBE_UNUSED, char c)
 {
   switch (c)
     {
@@ -148,8 +146,7 @@ put_string (format_data_t form, const char *s)
 }
 
 void
-put_int (format_data_t form _GL_UNUSED_PARAMETER,
-	 int argc, char *argv[], int nr)
+put_int (format_data_t form MAYBE_UNUSED, int argc, char *argv[], int nr)
 {
   char *fmt;
   if (argc > 0)
@@ -169,7 +166,7 @@ put_int (format_data_t form _GL_UNUSED_PARAMETER,
 	    p++;
 
 	  if ((*p == 'h' || *p == 'H') && p[1])
-	    ++p; /* Half length modifier, go to type specifier.  */
+	    ++p;		/* Half length modifier, go to type specifier.  */
 
 	  switch (*p)
 	    {
@@ -205,7 +202,7 @@ put_int (format_data_t form _GL_UNUSED_PARAMETER,
 }
 
 void
-put_ulong (format_data_t form _GL_UNUSED_PARAMETER,
+put_ulong (format_data_t form MAYBE_UNUSED,
 	   int argc, char *argv[], unsigned long value)
 {
   char *fmt;
@@ -346,8 +343,8 @@ put_flags (format_data_t form, int argc, char *argv[], int flags)
 }
 
 void
-put_flags_short (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-		 char *argv[] _GL_UNUSED_PARAMETER, int flags)
+put_flags_short (format_data_t form, int argc MAYBE_UNUSED,
+		 MAYBE_UNUSED char *argv[], int flags)
 {
   char buf[IF_FORMAT_FLAGS_BUFSIZE];
   if_format_flags (flags, buf, sizeof buf);
@@ -378,9 +375,8 @@ format_handler (const char *name, format_data_t form, int argc, char *argv[])
 }
 
 void
-fh_nothing (format_data_t form _GL_UNUSED_PARAMETER,
-	    int argc _GL_UNUSED_PARAMETER,
-	    char *argv[] _GL_UNUSED_PARAMETER)
+fh_nothing (format_data_t form MAYBE_UNUSED,
+	    int argc MAYBE_UNUSED, MAYBE_UNUSED char *argv[])
 {
 }
 
@@ -439,15 +435,15 @@ fh_foreachformat (format_data_t form, int argc, char *argv[])
 }
 
 void
-fh_newline (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-	    char *argv[] _GL_UNUSED_PARAMETER)
+fh_newline (format_data_t form, int argc MAYBE_UNUSED,
+	    MAYBE_UNUSED char *argv[])
 {
   put_char (form, '\n');
 }
 
 void
-fh_tabulator (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-	      char *argv[] _GL_UNUSED_PARAMETER)
+fh_tabulator (format_data_t form, int argc MAYBE_UNUSED,
+	      MAYBE_UNUSED char *argv[])
 {
   put_char (form, '\t');
 }
@@ -491,18 +487,19 @@ fh_ifdisplay_query (format_data_t form, int argc, char *argv[])
       int f = if_nameztoflag ("UP", &rev);
 
       n = f && ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) == 0;
-      if (n) {
-	unsigned int uflags = (unsigned short) form->ifr->ifr_flags;
+      if (n)
+	{
+	  unsigned int uflags = (unsigned short) form->ifr->ifr_flags;
 
 # ifdef ifr_flagshigh
-	uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
-# endif /* ifr_flagshigh */
+	  uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
+# endif/* ifr_flagshigh */
 
-	n = n && (f & uflags);
-      };
+	  n = n && (f & uflags);
+	};
     }
 #else
-  n = 1;	/* Display all of them.  */
+  n = 1;			/* Display all of them.  */
 #endif
   select_arg (form, argc, argv, !n);
 }
@@ -562,13 +559,13 @@ fh_exists_query (format_data_t form, int argc, char *argv[])
   if (argc > 0)
     {
       struct format_handle *fh;
-      int sel = 2; /* assume 2nd arg by default */
+      int sel = 2;		/* assume 2nd arg by default */
 
       for (fh = format_handles; fh->name; fh++)
 	{
 	  if (!strcmp (fh->name, argv[0]))
 	    {
-	      sel = 1; /* select 1st argument */
+	      sel = 1;		/* select 1st argument */
 	      break;
 	    }
 	}
@@ -614,15 +611,14 @@ fh_error (format_data_t form, int argc, char *argv[])
 }
 
 void
-fh_progname (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-	     char *argv[] _GL_UNUSED_PARAMETER)
+fh_progname (format_data_t form, int argc MAYBE_UNUSED,
+	     MAYBE_UNUSED char *argv[])
 {
   put_string (form, program_name);
 }
 
 void
-fh_exit (format_data_t form _GL_UNUSED_PARAMETER,
-	 int argc, char *argv[])
+fh_exit (format_data_t form MAYBE_UNUSED, int argc, char *argv[])
 {
   int err = 0;
 
@@ -633,8 +629,7 @@ fh_exit (format_data_t form _GL_UNUSED_PARAMETER,
 }
 
 void
-fh_name (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-	 char *argv[] _GL_UNUSED_PARAMETER)
+fh_name (format_data_t form, int argc MAYBE_UNUSED, MAYBE_UNUSED char *argv[])
 {
   put_string (form, form->name);
 }
@@ -646,15 +641,14 @@ fh_index_query (format_data_t form, int argc, char *argv[])
 }
 
 void
-fh_index (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-	  char *argv[] _GL_UNUSED_PARAMETER)
+fh_index (format_data_t form, int argc MAYBE_UNUSED,
+	  MAYBE_UNUSED char *argv[])
 {
   int indx = if_nametoindex (form->name);
 
   if (indx == 0)
     error (EXIT_FAILURE, errno,
-	   "No index number found for interface `%s'",
-	   form->name);
+	   "No index number found for interface `%s'", form->name);
   *column += printf ("%i", indx);
   had_output = 1;
 }
@@ -676,8 +670,7 @@ fh_addr (format_data_t form, int argc, char *argv[])
 #ifdef SIOCGIFADDR
   if (ioctl (form->sfd, SIOCGIFADDR, form->ifr) < 0)
     error (EXIT_FAILURE, errno,
-	   "SIOCGIFADDR failed for interface `%s'",
-	   form->ifr->ifr_name);
+	   "SIOCGIFADDR failed for interface `%s'", form->ifr->ifr_name);
   else
     put_addr (form, argc, argv, &form->ifr->ifr_addr);
 #else
@@ -703,8 +696,7 @@ fh_netmask (format_data_t form, int argc, char *argv[])
 #ifdef SIOCGIFNETMASK
   if (ioctl (form->sfd, SIOCGIFNETMASK, form->ifr) < 0)
     error (EXIT_FAILURE, errno,
-	   "SIOCGIFNETMASK failed for interface `%s'",
-	   form->ifr->ifr_name);
+	   "SIOCGIFNETMASK failed for interface `%s'", form->ifr->ifr_name);
   else
     put_addr (form, argc, argv, &form->ifr->ifr_netmask);
 #else
@@ -733,14 +725,15 @@ fh_brdaddr_query (format_data_t form, int argc, char *argv[])
   uflags = (unsigned short) form->ifr->ifr_flags;
 #  ifdef ifr_flagshigh
   uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
-#  endif /* ifr_flagshigh */
+#  endif
+  /* ifr_flagshigh */
 
   if ((f & uflags) == 0)
     {
       select_arg (form, argc, argv, 1);
       return;
     }
-# endif /* SIOCGIFFLAGS */
+# endif/* SIOCGIFFLAGS */
 
   if (ioctl (form->sfd, SIOCGIFBRDADDR, form->ifr) >= 0)
     select_arg (form, argc, argv, 0);
@@ -755,8 +748,7 @@ fh_brdaddr (format_data_t form, int argc, char *argv[])
 #ifdef SIOCGIFBRDADDR
   if (ioctl (form->sfd, SIOCGIFBRDADDR, form->ifr) < 0)
     error (EXIT_FAILURE, errno,
-	   "SIOCGIFBRDADDR failed for interface `%s'",
-	   form->ifr->ifr_name);
+	   "SIOCGIFBRDADDR failed for interface `%s'", form->ifr->ifr_name);
   else
     put_addr (form, argc, argv, &form->ifr->ifr_broadaddr);
 #else
@@ -785,14 +777,15 @@ fh_dstaddr_query (format_data_t form, int argc, char *argv[])
   uflags = (unsigned short) form->ifr->ifr_flags;
 #  ifdef ifr_flagshigh
   uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
-#  endif /* ifr_flagshigh */
+#  endif
+  /* ifr_flagshigh */
 
   if ((f & uflags) == 0)
     {
       select_arg (form, argc, argv, 1);
       return;
     }
-# endif /* SIOCGIFFLAGS */
+# endif/* SIOCGIFFLAGS */
 
   if (ioctl (form->sfd, SIOCGIFDSTADDR, form->ifr) >= 0)
     select_arg (form, argc, argv, 0);
@@ -807,8 +800,7 @@ fh_dstaddr (format_data_t form, int argc, char *argv[])
 #ifdef SIOCGIFDSTADDR
   if (ioctl (form->sfd, SIOCGIFDSTADDR, form->ifr) < 0)
     error (EXIT_FAILURE, errno,
-	   "SIOCGIFDSTADDR failed for interface `%s'",
-	   form->ifr->ifr_name);
+	   "SIOCGIFDSTADDR failed for interface `%s'", form->ifr->ifr_name);
   else
     put_addr (form, argc, argv, &form->ifr->ifr_dstaddr);
 #else
@@ -834,8 +826,7 @@ fh_mtu (format_data_t form, int argc, char *argv[])
 #ifdef SIOCGIFMTU
   if (ioctl (form->sfd, SIOCGIFMTU, form->ifr) < 0)
     error (EXIT_FAILURE, errno,
-	   "SIOCGIFMTU failed for interface `%s'",
-	   form->ifr->ifr_name);
+	   "SIOCGIFMTU failed for interface `%s'", form->ifr->ifr_name);
   else
     put_int (form, argc, argv, form->ifr->ifr_mtu);
 #else
@@ -845,7 +836,7 @@ fh_mtu (format_data_t form, int argc, char *argv[])
 }
 
 /* The portable behaviour is to display strictly positive
- * metrics, but to supress the default value naught.
+ * metrics, but to suppress the default value naught.
  */
 void
 fh_metric_query (format_data_t form, int argc, char *argv[])
@@ -865,8 +856,7 @@ fh_metric (format_data_t form, int argc, char *argv[])
 #ifdef SIOCGIFMETRIC
   if (ioctl (form->sfd, SIOCGIFMETRIC, form->ifr) < 0)
     error (EXIT_FAILURE, errno,
-	   "SIOCGIFMETRIC failed for interface `%s'",
-	   form->ifr->ifr_name);
+	   "SIOCGIFMETRIC failed for interface `%s'", form->ifr->ifr_name);
   else
     put_int (form, argc, argv, form->ifr->ifr_metric);
 #else
@@ -892,15 +882,14 @@ fh_flags (format_data_t form, int argc, char *argv[])
 #ifdef SIOCGIFFLAGS
   if (ioctl (form->sfd, SIOCGIFFLAGS, form->ifr) < 0)
     error (EXIT_FAILURE, errno,
-	   "SIOCGIFFLAGS failed for interface `%s'",
-	   form->ifr->ifr_name);
+	   "SIOCGIFFLAGS failed for interface `%s'", form->ifr->ifr_name);
   else
     {
       unsigned int uflags = (unsigned short) form->ifr->ifr_flags;
 
 # ifdef ifr_flagshigh
       uflags |= (unsigned short) form->ifr->ifr_flagshigh << 16;
-# endif /* ifr_flagshigh */
+# endif/* ifr_flagshigh */
 
       if (argc >= 1)
 	{
@@ -930,8 +919,8 @@ fh_media_query (format_data_t form, int argc, char *argv[])
 }
 
 void
-fh_media (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-	  char *argv[] _GL_UNUSED_PARAMETER)
+fh_media (format_data_t form, int argc MAYBE_UNUSED,
+	  MAYBE_UNUSED char *argv[])
 {
   /* Must be overridden by a system dependent implementation.  */
   put_string (form, "(not known)");
@@ -947,8 +936,8 @@ fh_status_query (format_data_t form, int argc, char *argv[])
 }
 
 void
-fh_status (format_data_t form, int argc _GL_UNUSED_PARAMETER,
-	   char *argv[] _GL_UNUSED_PARAMETER)
+fh_status (format_data_t form, int argc MAYBE_UNUSED,
+	   MAYBE_UNUSED char *argv[])
 {
   /* Must be overridden by a system dependent implementation.  */
   put_string (form, "(not known)");
@@ -1077,8 +1066,8 @@ print_interfaceX (format_data_t form, int quiet)
 	break;
 
       /* Look at next character.  If it is a '$' or '}', print that
-	 and skip the '$'.  If it is something else than '{', print
-	 both.  Otherwise enter substitution mode.  */
+         and skip the '$'.  If it is something else than '{', print
+         both.  Otherwise enter substitution mode.  */
       switch (*(++p))
 	{
 	default:
@@ -1101,7 +1090,7 @@ print_interfaceX (format_data_t form, int quiet)
       q = strchr (p, '}');
       if (!q)
 	{
-	  /* Without a following '}', no substitution at all can occure,
+	  /* Without a following '}', no substitution at all can occur,
 	     so just dump the string that is missing.  */
 	  p -= 2;
 	  put_string (form, p);
@@ -1136,8 +1125,7 @@ print_interfaceX (format_data_t form, int quiet)
 	  else
 	    {
 	      int argc = 0;
-	      char **argv;
-	      argv = alloca (strlen (q) / 2);
+	      char **argv = NULL;
 
 	      while (*p == '{')
 		{
@@ -1145,6 +1133,7 @@ print_interfaceX (format_data_t form, int quiet)
 		  form->format = p;
 		  print_interfaceX (form, 1);
 		  q = form->format;
+		  argv = xrealloc (argv, (argc + 1) * sizeof (char *));
 		  argv[argc] = xmalloc (q - p + 1);
 		  memcpy (argv[argc], p, q - p);
 		  argv[argc][q - p] = '\0';
@@ -1155,11 +1144,14 @@ print_interfaceX (format_data_t form, int quiet)
 		}
 
 	      format_handler (id, form, argc, argv);
-
-	      /* Clean up.  */
+	      if (argv != NULL)
+		{
+		  /* Clean up.  */
+		  while (--argc >= 0)
+		    free (argv[argc]);
+		  free (argv);
+		}
 	      form->format = p;
-	      while (--argc >= 0)
-		free (argv[argc]);
 	    }
 	}
     }
