@@ -1,6 +1,6 @@
 /* Traverse a file hierarchy.
 
-   Copyright (C) 2004-2023 Free Software Foundation, Inc.
+   Copyright (C) 2004-2025 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -233,7 +233,6 @@ static int      fts_safe_changedir (FTS *, FTSENT *, int, const char *)
 #endif
 
 #define ISDOT(a)        (a[0] == '.' && (!a[1] || (a[1] == '.' && !a[2])))
-#define STREQ(a, b)     (strcmp (a, b) == 0)
 
 #define CLR(opt)        (sp->fts_options &= ~(opt))
 #define ISSET(opt)      ((sp->fts_options & (opt)) != 0)
@@ -1018,11 +1017,18 @@ check_for_dir:
                       fts_assert (p->fts_statp->st_size == FTS_NO_STAT_REQUIRED);
                   }
 
+                /* Skip files with different device numbers when FTS_MOUNT
+                   is set.  */
+                if (ISSET (FTS_MOUNT) && p->fts_info != FTS_NS &&
+                    p->fts_level != FTS_ROOTLEVEL &&
+                    p->fts_statp->st_dev != sp->fts_dev)
+                      goto next;
+
                 if (p->fts_info == FTS_D)
                   {
-                    /* Now that P->fts_statp is guaranteed to be valid,
-                       if this is a command-line directory, record its
-                       device number, to be used for FTS_XDEV.  */
+                    /* Now that P->fts_statp is guaranteed to be valid, if
+                       this is a command-line directory, record its device
+                       number, to be used for FTS_MOUNT and FTS_XDEV.  */
                     if (p->fts_level == FTS_ROOTLEVEL)
                       sp->fts_dev = p->fts_statp->st_dev;
                     Dprintf (("  entering: %s\n", p->fts_path));
@@ -1344,9 +1350,9 @@ fts_build (register FTS *sp, int type)
                   cur->fts_info = FTS_D;
                 else if (! enter_dir (sp, cur))
                   {
-                    int err = errno;
+                    int saved_errno = errno;
                     closedir_and_clear (cur->fts_dirp);
-                    __set_errno (err);
+                    __set_errno (saved_errno);
                     return NULL;
                   }
               }
@@ -1443,6 +1449,13 @@ fts_build (register FTS *sp, int type)
                 __set_errno (0);
                 struct dirent *dp = readdir(cur->fts_dirp);
                 if (dp == NULL) {
+                        /* Some readdir()s do not absorb ENOENT (dir
+                           deleted but open).  This bug was fixed in
+                           glibc 2.3 (2002).  */
+#if ! (2 < __GLIBC__ + (3 <= __GLIBC_MINOR__))
+                        if (errno == ENOENT)
+                          errno = 0;
+#endif
                         if (errno) {
                                 cur->fts_errno = errno;
                                 /* If we've not read any items yet, treat
@@ -1523,19 +1536,21 @@ mem1:                           saved_errno = errno;
                            entry. In many cases, it will simply fts_stat it,
                            but we can take advantage of any d_type information
                            to optimize away the unnecessary stat calls.  I.e.,
-                           if FTS_NOSTAT is in effect and we're not following
-                           symlinks (FTS_PHYSICAL) and d_type indicates this
-                           is *not* a directory, then we won't have to stat it
-                           at all.  If it *is* a directory, then (currently)
-                           we stat it regardless, in order to get device and
-                           inode numbers.  Some day we might optimize that
-                           away, too, for directories where d_ino is known to
-                           be valid.  */
+                           if FTS_NOSTAT is in effect, we don't need device
+                           numbers unconditionally (FTS_MOUNT) and we're not
+                           following symlinks (FTS_PHYSICAL) and d_type
+                           indicates this is *not* a directory, then we won't
+                           have to stat it  at all.  If it *is* a directory,
+                           then (currently) we stat it regardless, in order to
+                           get device and inode numbers.  Some day we might
+                           optimize that away, too, for directories where
+                           d_ino is known to be valid.  */
                         bool skip_stat = (ISSET(FTS_NOSTAT)
                                           && DT_IS_KNOWN(dp)
                                           && ! DT_MUST_BE(dp, DT_DIR)
                                           && (ISSET(FTS_PHYSICAL)
-                                              || ! DT_MUST_BE(dp, DT_LNK)));
+                                              || ! DT_MUST_BE(dp, DT_LNK))
+                                          && ! ISSET(FTS_MOUNT));
                         p->fts_info = FTS_NSOK;
                         /* Propagate dirent.d_type information back
                            to caller, when possible.  */
@@ -1654,8 +1669,9 @@ getdevino (int fd)
 static void
 find_matching_ancestor (FTSENT const *e_curr, struct Active_dir const *ad)
 {
-  FTSENT const *ent;
-  for (ent = e_curr; ent->fts_level >= FTS_ROOTLEVEL; ent = ent->fts_parent)
+  for (FTSENT const *ent = e_curr;
+       ent->fts_level >= FTS_ROOTLEVEL;
+       ent = ent->fts_parent)
     {
       if (ad->ino == ent->fts_statp->st_ino
           && ad->dev == ent->fts_statp->st_dev)
@@ -1663,8 +1679,9 @@ find_matching_ancestor (FTSENT const *e_curr, struct Active_dir const *ad)
     }
   printf ("ERROR: tree dir, %s, not active\n", ad->fts_ent->fts_accpath);
   printf ("active dirs:\n");
-  for (ent = e_curr;
-       ent->fts_level >= FTS_ROOTLEVEL; ent = ent->fts_parent)
+  for (FTSENT const *ent = e_curr;
+       ent->fts_level >= FTS_ROOTLEVEL;
+       ent = ent->fts_parent)
     printf ("  %s(%"PRIuMAX"/%"PRIuMAX") to %s(%"PRIuMAX"/%"PRIuMAX")...\n",
             ad->fts_ent->fts_accpath,
             (uintmax_t) ad->dev,
@@ -1678,13 +1695,14 @@ void
 fts_cross_check (FTS const *sp)
 {
   FTSENT const *ent = sp->fts_cur;
-  FTSENT const *t;
   if ( ! ISSET (FTS_TIGHT_CYCLE_CHECK))
     return;
 
   Dprintf (("fts-cross-check cur=%s\n", ent->fts_path));
   /* Make sure every parent dir is in the tree.  */
-  for (t = ent->fts_parent; t->fts_level >= FTS_ROOTLEVEL; t = t->fts_parent)
+  for (FTSENT const *t = ent->fts_parent;
+       t->fts_level >= FTS_ROOTLEVEL;
+       t = t->fts_parent)
     {
       struct Active_dir ad;
       ad.ino = t->fts_statp->st_ino;
@@ -1698,14 +1716,12 @@ fts_cross_check (FTS const *sp)
   if (ent->fts_parent->fts_level >= FTS_ROOTLEVEL
       && (ent->fts_info == FTS_DP
           || ent->fts_info == FTS_D))
-    {
-      struct Active_dir *ad;
-      for (ad = hash_get_first (sp->fts_cycle.ht); ad != NULL;
-           ad = hash_get_next (sp->fts_cycle.ht, ad))
-        {
-          find_matching_ancestor (ent, ad);
-        }
-    }
+    for (struct Active_dir *ad = hash_get_first (sp->fts_cycle.ht);
+         ad != NULL;
+         ad = hash_get_next (sp->fts_cycle.ht, ad))
+      {
+        find_matching_ancestor (ent, ad);
+      }
 }
 
 static bool
@@ -1858,13 +1874,13 @@ fts_sort (FTS *sp, FTSENT *head, register size_t nitems)
            run-time representation, and one can convert sp->fts_compar to
            the type qsort expects without problem.  Use the heuristic that
            this is OK if the two pointer types are the same size, and if
-           converting FTSENT ** to long int is the same as converting
-           FTSENT ** to void * and then to long int.  This heuristic isn't
+           converting FTSENT ** to uintptr_t is the same as converting
+           FTSENT ** to void * and then to uintptr_t.  This heuristic isn't
            valid in general but we don't know of any counterexamples.  */
         FTSENT *dummy;
         int (*compare) (void const *, void const *) =
           ((sizeof &dummy == sizeof (void *)
-            && (long int) &dummy == (long int) (void *) &dummy)
+            && (uintptr_t) &dummy == (uintptr_t) (void *) &dummy)
            ? (int (*) (void const *, void const *)) sp->fts_compar
            : fts_compar);
 
@@ -1934,7 +1950,7 @@ internal_function
 fts_lfree (register FTSENT *head)
 {
         register FTSENT *p;
-        int err = errno;
+        int saved_errno = errno;
 
         /* Free a linked list of structures. */
         while ((p = head)) {
@@ -1944,7 +1960,7 @@ fts_lfree (register FTSENT *head)
                 free(p);
         }
 
-        __set_errno (err);
+        __set_errno (saved_errno);
 }
 
 /*
@@ -2040,8 +2056,9 @@ static int
 internal_function
 fts_safe_changedir (FTS *sp, FTSENT *p, int fd, char const *dir)
 {
+        fts_assert (0 <= fd || dir != NULL);
         int ret;
-        bool is_dotdot = dir && STREQ (dir, "..");
+        bool is_dotdot = dir && streq (dir, "..");
         int newfd;
 
         /* This clause handles the unusual case in which FTS_NOCHDIR
@@ -2087,7 +2104,7 @@ fts_safe_changedir (FTS *sp, FTSENT *p, int fd, char const *dir)
            not "..", diropen's use of O_NOFOLLOW ensures we don't mistakenly
            follow a symlink, so we can avoid the expense of this fstat.  */
         if (ISSET(FTS_LOGICAL) || ! HAVE_WORKING_O_NOFOLLOW
-            || (dir && STREQ (dir, "..")))
+            || (dir && streq (dir, "..")))
           {
             struct stat sb;
             if (fstat(newfd, &sb))
