@@ -1,27 +1,22 @@
 /* Emulation for poll(2)
    Contributed by Paolo Bonzini.
 
-   Copyright 2001-2003, 2006-2015 Free Software Foundation, Inc.
+   Copyright 2001-2003, 2006-2025 Free Software Foundation, Inc.
 
    This file is part of gnulib.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+   This file is free software: you can redistribute it and/or modify
+   it under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of the
+   License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU Lesser General Public License for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with this program; if not, see <http://www.gnu.org/licenses/>.  */
-
-/* Tell gcc not to warn about the (nfd < 0) tests, below.  */
-#if (__GNUC__ == 4 && 3 <= __GNUC_MINOR__) || 4 < __GNUC__
-# pragma GCC diagnostic ignored "-Wtype-limits"
-#endif
+   You should have received a copy of the GNU Lesser General Public License
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 #include <alloca.h>
@@ -34,14 +29,18 @@
 #include <errno.h>
 #include <limits.h>
 
-#if (defined _WIN32 || defined __WIN32__) && ! defined __CYGWIN__
+#if defined _WIN32 && ! defined __CYGWIN__
 # define WINDOWS_NATIVE
 # include <winsock2.h>
 # include <windows.h>
 # include <io.h>
 # include <stdio.h>
 # include <conio.h>
-# include "msvc-nothrow.h"
+# if GNULIB_MSVC_NOTHROW
+#  include "msvc-nothrow.h"
+# else
+#  include <io.h>
+# endif
 #else
 # include <sys/time.h>
 # include <unistd.h>
@@ -72,6 +71,41 @@
 
 #ifdef WINDOWS_NATIVE
 
+/* Don't assume that UNICODE is not defined.  */
+# undef GetModuleHandle
+# define GetModuleHandle GetModuleHandleA
+# undef PeekConsoleInput
+# define PeekConsoleInput PeekConsoleInputA
+# undef CreateEvent
+# define CreateEvent CreateEventA
+# undef PeekMessage
+# define PeekMessage PeekMessageA
+# undef DispatchMessage
+# define DispatchMessage DispatchMessageA
+
+/* Do *not* use the function WSAPoll
+   <https://docs.microsoft.com/en-us/windows/desktop/api/winsock2/nf-winsock2-wsapoll>
+   because there is a bug named “Windows 8 Bugs 309411 - WSAPoll does not
+   report failed connections” that Microsoft won't fix.
+   See Daniel Stenberg: "WASPoll is broken"
+   <https://daniel.haxx.se/blog/2012/10/10/wsapoll-is-broken/>.  */
+
+/* Here we need the recv() function from Windows, that takes a SOCKET as
+   first argument, not any possible gnulib override.  */
+# undef recv
+
+/* Here we need the select() function from Windows, because we pass bit masks
+   of SOCKETs, not bit masks of FDs.  */
+# undef select
+
+/* Here we need timeval from Windows since this is what the select() function
+   from Windows requires.  */
+# undef timeval
+
+/* Avoid warnings from gcc -Wcast-function-type.  */
+# define GetProcAddress \
+   (void *) GetProcAddress
+
 static BOOL IsConsoleHandle (HANDLE h)
 {
   DWORD mode;
@@ -133,7 +167,7 @@ typedef DWORD (WINAPI *PNtQueryInformationFile)
 static int
 windows_compute_revents (HANDLE h, int *p_sought)
 {
-  int i, ret, happened;
+  int ret, happened;
   INPUT_RECORD *irbuffer;
   DWORD avail, nbuffer;
   BOOL bRet;
@@ -206,7 +240,7 @@ windows_compute_revents (HANDLE h, int *p_sought)
           if (!bRet || avail == 0)
             return POLLHUP;
 
-          for (i = 0; i < avail; i++)
+          for (int i = 0; i < avail; i++)
             if (irbuffer[i].EventType == KEY_EVENT)
               return *p_sought;
           return 0;
@@ -335,13 +369,13 @@ poll (struct pollfd *pfd, nfds_t nfd, int timeout)
   int maxfd, rc;
   nfds_t i;
 
-  if (nfd < 0)
+  if (nfd > INT_MAX)
     {
       errno = EINVAL;
       return -1;
     }
-  /* Don't check directly for NFD too large.  Any practical use of a
-     too-large NFD is caught by one of the other checks below, and
+  /* Don't check directly for NFD greater than OPEN_MAX.  Any practical use
+     of a too-large NFD is caught by one of the other checks below, and
      checking directly for getdtablesize is too much of a portability
      and/or performance and/or correctness hassle.  */
 
@@ -357,14 +391,15 @@ poll (struct pollfd *pfd, nfds_t nfd, int timeout)
   if (timeout == 0)
     {
       ptv = &tv;
-      ptv->tv_sec = 0;
-      ptv->tv_usec = 0;
+      tv = (struct timeval) {0};
     }
   else if (timeout > 0)
     {
       ptv = &tv;
-      ptv->tv_sec = timeout / 1000;
-      ptv->tv_usec = (timeout % 1000) * 1000;
+      tv = (struct timeval) {
+        .tv_sec = timeout / 1000,
+        .tv_usec = (timeout % 1000) * 1000
+      };
     }
   else if (timeout == INFTIM)
     /* wait forever */
@@ -380,28 +415,29 @@ poll (struct pollfd *pfd, nfds_t nfd, int timeout)
   FD_ZERO (&rfds);
   FD_ZERO (&wfds);
   FD_ZERO (&efds);
-  for (i = 0; i < nfd; i++)
+  for (int i = 0; i < nfd; i++)
     {
-      if (pfd[i].fd < 0)
-        continue;
-      if (maxfd < pfd[i].fd)
+      if (pfd[i].fd >= 0)
         {
-          maxfd = pfd[i].fd;
-          if (FD_SETSIZE <= maxfd)
+          if (maxfd < pfd[i].fd)
             {
-              errno = EINVAL;
-              return -1;
+              maxfd = pfd[i].fd;
+              if (FD_SETSIZE <= maxfd)
+                {
+                  errno = EINVAL;
+                  return -1;
+                }
             }
+          if (pfd[i].events & (POLLIN | POLLRDNORM))
+            FD_SET (pfd[i].fd, &rfds);
+          /* see select(2): "the only exceptional condition detectable
+             is out-of-band data received on a socket", hence we push
+             POLLWRBAND events onto wfds instead of efds. */
+          if (pfd[i].events & (POLLOUT | POLLWRNORM | POLLWRBAND))
+            FD_SET (pfd[i].fd, &wfds);
+          if (pfd[i].events & (POLLPRI | POLLRDBAND))
+            FD_SET (pfd[i].fd, &efds);
         }
-      if (pfd[i].events & (POLLIN | POLLRDNORM))
-        FD_SET (pfd[i].fd, &rfds);
-      /* see select(2): "the only exceptional condition detectable
-         is out-of-band data received on a socket", hence we push
-         POLLWRBAND events onto wfds instead of efds. */
-      if (pfd[i].events & (POLLOUT | POLLWRNORM | POLLWRBAND))
-        FD_SET (pfd[i].fd, &wfds);
-      if (pfd[i].events & (POLLPRI | POLLRDBAND))
-        FD_SET (pfd[i].fd, &efds);
     }
 
   /* examine fd sets */
@@ -411,7 +447,7 @@ poll (struct pollfd *pfd, nfds_t nfd, int timeout)
 
   /* establish results */
   rc = 0;
-  for (i = 0; i < nfd; i++)
+  for (int i = 0; i < nfd; i++)
     {
       pfd[i].revents = (pfd[i].fd < 0
                         ? 0
@@ -431,9 +467,8 @@ poll (struct pollfd *pfd, nfds_t nfd, int timeout)
   BOOL poll_again;
   MSG msg;
   int rc = 0;
-  nfds_t i;
 
-  if (nfd < 0 || timeout < -1)
+  if (nfd > INT_MAX || timeout < -1)
     {
       errno = EINVAL;
       return -1;
@@ -450,53 +485,52 @@ restart:
   FD_ZERO (&xfds);
 
   /* Classify socket handles and create fd sets. */
-  for (i = 0; i < nfd; i++)
+  for (nfds_t i = 0; i < nfd; i++)
     {
       int sought = pfd[i].events;
       pfd[i].revents = 0;
-      if (pfd[i].fd < 0)
-        continue;
-      if (!(sought & (POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM | POLLWRBAND
-                      | POLLPRI | POLLRDBAND)))
-        continue;
-
-      h = (HANDLE) _get_osfhandle (pfd[i].fd);
-      assure (h != NULL);
-      if (IsSocketHandle (h))
+      if (pfd[i].fd >= 0
+          && (sought & (POLLIN | POLLRDNORM | POLLOUT | POLLWRNORM | POLLWRBAND
+                        | POLLPRI | POLLRDBAND)))
         {
-          int requested = FD_CLOSE;
+          h = (HANDLE) _get_osfhandle (pfd[i].fd);
+          assure (h != NULL);
+          if (IsSocketHandle (h))
+            {
+              int requested = FD_CLOSE;
 
-          /* see above; socket handles are mapped onto select.  */
-          if (sought & (POLLIN | POLLRDNORM))
-            {
-              requested |= FD_READ | FD_ACCEPT;
-              FD_SET ((SOCKET) h, &rfds);
-            }
-          if (sought & (POLLOUT | POLLWRNORM | POLLWRBAND))
-            {
-              requested |= FD_WRITE | FD_CONNECT;
-              FD_SET ((SOCKET) h, &wfds);
-            }
-          if (sought & (POLLPRI | POLLRDBAND))
-            {
-              requested |= FD_OOB;
-              FD_SET ((SOCKET) h, &xfds);
-            }
+              /* see above; socket handles are mapped onto select.  */
+              if (sought & (POLLIN | POLLRDNORM))
+                {
+                  requested |= FD_READ | FD_ACCEPT;
+                  FD_SET ((SOCKET) h, &rfds);
+                }
+              if (sought & (POLLOUT | POLLWRNORM | POLLWRBAND))
+                {
+                  requested |= FD_WRITE | FD_CONNECT;
+                  FD_SET ((SOCKET) h, &wfds);
+                }
+              if (sought & (POLLPRI | POLLRDBAND))
+                {
+                  requested |= FD_OOB;
+                  FD_SET ((SOCKET) h, &xfds);
+                }
 
-          if (requested)
-            WSAEventSelect ((SOCKET) h, hEvent, requested);
-        }
-      else
-        {
-          /* Poll now.  If we get an event, do not poll again.  Also,
-             screen buffer handles are waitable, and they'll block until
-             a character is available.  windows_compute_revents eliminates
-             bits for the "wrong" direction. */
-          pfd[i].revents = windows_compute_revents (h, &sought);
-          if (sought)
-            handle_array[nhandles++] = h;
-          if (pfd[i].revents)
-            timeout = 0;
+              if (requested)
+                WSAEventSelect ((SOCKET) h, hEvent, requested);
+            }
+          else
+            {
+              /* Poll now.  If we get an event, do not poll again.  Also,
+                 screen buffer handles are waitable, and they'll block until
+                 a character is available.  windows_compute_revents eliminates
+                 bits for the "wrong" direction. */
+              pfd[i].revents = windows_compute_revents (h, &sought);
+              if (sought)
+                handle_array[nhandles++] = h;
+              if (pfd[i].revents)
+                timeout = 0;
+            }
         }
     }
 
@@ -541,46 +575,45 @@ restart:
   /* Place a sentinel at the end of the array.  */
   handle_array[nhandles] = NULL;
   nhandles = 1;
-  for (i = 0; i < nfd; i++)
+  for (nfds_t i = 0; i < nfd; i++)
     {
       int happened;
 
-      if (pfd[i].fd < 0)
-        continue;
-      if (!(pfd[i].events & (POLLIN | POLLRDNORM |
-                             POLLOUT | POLLWRNORM | POLLWRBAND)))
-        continue;
-
-      h = (HANDLE) _get_osfhandle (pfd[i].fd);
-      if (h != handle_array[nhandles])
+      if (pfd[i].fd >= 0
+          && (pfd[i].events & (POLLIN | POLLRDNORM |
+                               POLLOUT | POLLWRNORM | POLLWRBAND)))
         {
-          /* It's a socket.  */
-          WSAEnumNetworkEvents ((SOCKET) h, NULL, &ev);
-          WSAEventSelect ((SOCKET) h, 0, 0);
+          h = (HANDLE) _get_osfhandle (pfd[i].fd);
+          if (h != handle_array[nhandles])
+            {
+              /* It's a socket.  */
+              WSAEnumNetworkEvents ((SOCKET) h, NULL, &ev);
+              WSAEventSelect ((SOCKET) h, 0, 0);
 
-          /* If we're lucky, WSAEnumNetworkEvents already provided a way
-             to distinguish FD_READ and FD_ACCEPT; this saves a recv later.  */
-          if (FD_ISSET ((SOCKET) h, &rfds)
-              && !(ev.lNetworkEvents & (FD_READ | FD_ACCEPT)))
-            ev.lNetworkEvents |= FD_READ | FD_ACCEPT;
-          if (FD_ISSET ((SOCKET) h, &wfds))
-            ev.lNetworkEvents |= FD_WRITE | FD_CONNECT;
-          if (FD_ISSET ((SOCKET) h, &xfds))
-            ev.lNetworkEvents |= FD_OOB;
+              /* If we're lucky, WSAEnumNetworkEvents already provided a way
+                 to distinguish FD_READ and FD_ACCEPT; this saves a recv later.  */
+              if (FD_ISSET ((SOCKET) h, &rfds)
+                  && !(ev.lNetworkEvents & (FD_READ | FD_ACCEPT)))
+                ev.lNetworkEvents |= FD_READ | FD_ACCEPT;
+              if (FD_ISSET ((SOCKET) h, &wfds))
+                ev.lNetworkEvents |= FD_WRITE | FD_CONNECT;
+              if (FD_ISSET ((SOCKET) h, &xfds))
+                ev.lNetworkEvents |= FD_OOB;
 
-          happened = windows_compute_revents_socket ((SOCKET) h, pfd[i].events,
-                                                     ev.lNetworkEvents);
+              happened = windows_compute_revents_socket ((SOCKET) h, pfd[i].events,
+                                                         ev.lNetworkEvents);
+            }
+          else
+            {
+              /* Not a socket.  */
+              int sought = pfd[i].events;
+              happened = windows_compute_revents (h, &sought);
+              nhandles++;
+            }
+
+          if ((pfd[i].revents |= happened) != 0)
+            rc++;
         }
-      else
-        {
-          /* Not a socket.  */
-          int sought = pfd[i].events;
-          happened = windows_compute_revents (h, &sought);
-          nhandles++;
-        }
-
-       if ((pfd[i].revents |= happened) != 0)
-        rc++;
     }
 
   if (!rc && timeout == INFTIM)
