@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011, 2012 Free Software Foundation, Inc.
+  Copyright (C) 2001-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -23,9 +22,9 @@
 #include <unistd.h>
 
 unsigned short
-icmp_cksum (unsigned char * addr, int len)
+icmp_cksum (unsigned char *addr, int len)
 {
-  register int sum = 0;
+  int sum = 0;
   unsigned short answer = 0;
   unsigned short *wp;
 
@@ -35,7 +34,7 @@ icmp_cksum (unsigned char * addr, int len)
   /* Take in an odd byte if present */
   if (len == 1)
     {
-      *(unsigned char *) & answer = *(unsigned char *) wp;
+      *(unsigned char *) &answer = *(unsigned char *) wp;
       sum += answer;
     }
 
