@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-  2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software
-  Foundation, Inc.
+  Copyright (C) 1996-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -55,6 +53,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <attribute.h>
 
 /*
   According to Unix-FAQ maintained by Andrew Gierth:
@@ -87,7 +86,7 @@
   we inherited from our parent process. We have no way of knowing where
   these fds might have been redirected to. Note that many daemons use
   sysconf() to determine the limit _SC_OPEN_MAX. _SC_OPEN_MAX tells you the
-  maximun open files/process. Then in a loop, the daemon can close all
+  maximum open files/process. Then in a loop, the daemon can close all
   possible file descriptors. You have to decide if you need to do this or not.
   If you think that there might be file-descriptors open you should close
   them, since there's a limit on number of concurrent file descriptors.
@@ -103,7 +102,7 @@
 #define MAXFD 64
 
 void
-waitdaemon_timeout (int signo)
+waitdaemon_timeout (int signo MAYBE_UNUSED)
 {
   int left;
 
@@ -141,14 +140,16 @@ waitdaemon (int nochdir, int noclose, int maxwait)
 	  alarm (maxwait);
 	  pause ();
 	}
-      _exit (0);
+      _exit (EXIT_SUCCESS);
     }
 
   if (setsid () == -1)
     return -1;
 
-  /* SIGHUP is ignore because when the session leader terminates
-     all process in the session (the second child) are sent the SIGHUP.  */
+  /* SIGHUP must be ignored, because when the session leader terminates,
+     then SIGHUP is sent to all process belonging to the same session,
+     i.e., also to the second child.
+   */
   signal (SIGHUP, SIG_IGN);
 
   switch (fork ())
@@ -160,18 +161,18 @@ waitdaemon (int nochdir, int noclose, int maxwait)
       return -1;
 
     default:
-      _exit (0);
+      _exit (EXIT_SUCCESS);
     }
 
-  if (!nochdir)
-    chdir ("/");
+  if (!nochdir && chdir ("/") < 0)
+    return -1;			/* Unlikely failure, but check it.  */
 
   if (!noclose)
     {
       int i;
       long fdlimit = -1;
 
-#if defined HAVE_SYSCONF && defined _SC_OPEN_MAX
+#if defined _SC_OPEN_MAX
       fdlimit = sysconf (_SC_OPEN_MAX);
 #elif defined (HAVE_GETDTABLESIZE)
       fdlimit = getdtablesize ();
