@@ -1,6 +1,5 @@
 /* shishi.c -- functions to use kerberos V with shishi
-  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free
-  Software Foundation, Inc.
+  Copyright (C) 2005-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -28,11 +27,11 @@
 # include <unistd.h>
 # include <shishi_def.h>
 
-/* shishi authentication */
+/* shishi authentication, client side */
 int
-shishi_auth (Shishi ** handle, int verbose, char **cname,
-	     const char *sname, int sock,
-	     char *cmd, int port, Shishi_key ** enckey, char *realm)
+shishi_auth (Shishi **handle, int verbose, char **cname,
+	     const char *sname, int sock, char *cmd,
+	     unsigned short port, Shishi_key **enckey, const char *realm)
 {
   Shishi_ap *ap;
   Shishi_tkt *tkt;
@@ -40,8 +39,8 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
   Shishi *h;
 
   int rc;
-  char *out;
-  int outlen;
+  char *out, *p;
+  size_t outlen;
   int krb5len, msglen;
   char *tmpserver;
   char auth;
@@ -57,20 +56,20 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
     {
       fprintf (stderr, "shishi_check_version() failed:\n"
 	       "Header file incompatible with shared library.\n");
-      return 1;
+      return SHISHI_INVALID_ARGUMENT;
     }
 
-  if (realm)
-    shishi_realm_default_set (*handle, realm);
-
   rc = shishi_init (handle);
   if (rc != SHISHI_OK)
     {
       fprintf (stderr,
 	       "error initializing shishi: %s\n", shishi_strerror (rc));
-      return 1;
+      return rc;
     }
 
+  if (realm)
+    shishi_realm_default_set (*handle, realm);
+
   h = *handle;
 
   if (!*cname)
@@ -93,11 +92,18 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
   read (sock, &auth, 1);
   if (auth)
     {
-      read (sock, errormsg, 100);
-      errormsg[100] = '\0';
+      ssize_t n;
+
+      errormsg[0] = '\0';
+      n = read (sock, errormsg, sizeof (errormsg) - 1);
+
+      if (n >= 0 && n < (ssize_t) sizeof (errormsg))
+	errormsg[n] = '\0';
+      else
+	errormsg[sizeof (errormsg) - 1] = '\0';
 
       fprintf (stderr, "Error during server authentication : %s\n", errormsg);
-      return 1;
+      return SHISHI_VERIFY_FAILED;
     }
 
   if (verbose)
@@ -114,11 +120,27 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
   if (!tmpserver)
     {
       perror ("shishi_auth()");
-      return 1;
+      return SHISHI_TOO_SMALL_BUFFER;
+    }
+
+  p = strchr (sname, '/');
+  if (p && (p != sname))
+    strcpy (tmpserver, sname);	/* Non-empty prefix.  */
+  else
+    sprintf (tmpserver, "%s/%s", SERVICE, sname + (p ? 1 : 0));
+
+  /* Retrieve realm assigned to this server as per configuration,
+   * unless an explicit domain was passed in the call.
+   */
+  if (!realm)
+    {
+      if (!p)
+	p = (char *) sname;
+      else if (*p == '/')
+	++p;
+
+      shishi_realm_default_set (h, shishi_realm_for_server (h, p));
     }
-  strcpy (tmpserver, SERVICE);
-  strcat (tmpserver, "/");
-  strcat (tmpserver, sname);
 
   hint.client = (char *) *cname;
   hint.server = (char *) tmpserver;
@@ -126,9 +148,10 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
   tkt = shishi_tkts_get (shishi_tkts_default (h), &hint);
   if (!tkt)
     {
+      fprintf (stderr, "cannot find ticket for \"%s@%s\"\n",
+	       tmpserver, shishi_realm_default (h));
       free (tmpserver);
-      fprintf (stderr, "cannot find ticket for \"%s\"\n", sname);
-      return 1;
+      return SHISHI_INVALID_TICKET;
     }
 
   free (tmpserver);
@@ -142,13 +165,14 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
   if (rc != SHISHI_OK)
     {
       fprintf (stderr, "cannot create authentication context\n");
-      return 1;
+      return rc;
     }
 
 
   /* checksum = port: terminal name */
 
-  snprintf (cksumdata, 100, "%u:%s%s", ntohs (port), cmd, *cname);
+  snprintf (cksumdata, sizeof (cksumdata) - 1,
+	    "%u:%s%s", ntohs (port), cmd, *cname);
 
   /* add checksum to authenticator */
 
@@ -164,7 +188,7 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
       fprintf (stderr, "cannot build authentication request: %s\n",
 	       shishi_strerror (rc));
 
-      return 1;
+      return rc;
     }
 
   if (verbose)
@@ -177,16 +201,19 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
 
   /* send size of AP-REQ to the server */
 
-  msglen = htonl (outlen);
+  msglen = outlen;
+  msglen = htonl (msglen);
   write (sock, (char *) &msglen, sizeof (int));
 
   /* send AP-REQ to the server */
 
   write (sock, out, outlen);
 
-  /* read a respond from server - what ? */
+  /* read response from server - what ? */
 
-  read (sock, &auth, sizeof (int));
+  read (sock, &rc, sizeof (rc));
+  if (rc)
+    return SHISHI_APREP_VERIFY_FAILED;
 
   /* For mutual authentication, wait for server reply. */
 
@@ -197,10 +224,10 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
 
       /* read size of the AP-REP */
 
-      read (sock, (char *) &outlen, sizeof (int));
+      read (sock, (char *) &msglen, sizeof (int));
 
       /* read AP-REP */
-      outlen = ntohl (outlen);
+      outlen = ntohl (msglen);
       outlen = read (sock, out, outlen);
 
       rc = shishi_ap_rep_verify_der (ap, out, outlen);
@@ -216,7 +243,7 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
 	  else
 	    fprintf (stderr, "AP-REP verification error: %s\n",
 		     shishi_strerror (rc));
-	  return 1;
+	  return rc;
 	}
 
       /* The server is authenticated. */
@@ -228,38 +255,41 @@ shishi_auth (Shishi ** handle, int verbose, char **cname,
   if (verbose)
     printf ("User authenticated.\n");
 
-  return 0;
+  return SHISHI_OK;
 
 }
 
+/*
+ * XXX: Is this ever needed?
+ *
 static void
 senderror (int s, char type, char *buf)
 {
   write (s, &type, sizeof (char));
   write (s, buf, strlen (buf));
 }
+*/
 
+/* shishi authentication, server side */
 int
-get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
-	  Shishi_key ** enckey, const char **err_msg, int *protoversion,
-	  int *cksumtype, char **cksum, int *cksumlen)
+get_auth (int infd, Shishi **handle, Shishi_ap **ap,
+	  Shishi_key **enckey, const char **err_msg, int *protoversion,
+	  int *cksumtype, char **cksum, size_t *cksumlen, char *srvname)
 {
   Shishi_key *key;
   char *out;
-  int outlen;
+  size_t outlen;
   char *buf;
   int buflen;
   int len;
   int rc;
-  int i;
   int error;
   /* KERBEROS 5 SENDAUTH MESSAGE */
   char krb5sendauth[] = "KRB5_SENDAUTH_V1.0";
   /* PROTOCOL VERSION */
   char krb5kcmd1[] = "KCMDV0.1";
   char krb5kcmd2[] = "KCMDV0.2";
-  int auth_correct = 0;
-  char *servername;
+  char *servername, *server = NULL, *realm = NULL;
 
   *err_msg = NULL;
   /* Get key for the server. */
@@ -268,20 +298,67 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
     {
       *err_msg =
 	"shishi_check_version() failed: header file incompatible with shared library.";
-      return 1;
+      return SHISHI_INVALID_ARGUMENT;
     }
 
   rc = shishi_init_server (handle);
   if (rc != SHISHI_OK)
     return rc;
 
-  servername = shishi_server_for_local_service (*handle, SERVICE);
+  if (srvname && *srvname)
+    {
+      rc = shishi_parse_name (*handle, srvname, &server, &realm);
+      if (rc != SHISHI_OK)
+	{
+	  *err_msg = shishi_strerror (rc);
+	  return rc;
+	}
+    }
 
-  key = shishi_hostkeys_for_server (*handle, servername);
+  if (server && *server)
+    {
+      char *p;
+
+      servername = malloc (sizeof (SERVICE) + strlen (server) + 2);
+      if (!servername)
+	{
+	  *err_msg = "Not enough memory";
+	  return SHISHI_TOO_SMALL_BUFFER;
+	}
+
+      p = strchr (server, '/');
+      if (p && (p != server))
+	sprintf (servername, "%s", server);	/* Non-empty prefix.  */
+      else
+	sprintf (servername, "%s/%s", SERVICE, server + (p ? 1 : 0));	/* Remove initial slash.  */
+    }
+  else
+    servername = shishi_server_for_local_service (*handle, SERVICE);
+
+  if (realm && *realm)
+    shishi_realm_default_set (*handle, realm);
+
+  free (server);
+  free (realm);
+
+  /* Enable use of `~/.k5login'.  */
+  if (shishi_check_version ("1.0.2"))	/* Faulty in version 1.0.1.  */
+    {
+      rc = shishi_cfg_authorizationtype_set (*handle, "k5login basic");
+      if (rc != SHISHI_OK)
+	{
+	  *err_msg = shishi_error (*handle);
+	  return rc;
+	}
+    }
+
+  key = shishi_hostkeys_for_serverrealm (*handle, servername,
+					 shishi_realm_default (*handle));
+  free (servername);
   if (!key)
     {
       *err_msg = shishi_error (*handle);
-      return 1;
+      return SHISHI_INVALID_KEY;
     }
 
   /* Read Kerberos 5 sendauth message */
@@ -289,7 +366,7 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
   if (rc != sizeof (int))
     {
       *err_msg = "Error reading message size";
-      return 1;
+      return SHISHI_IO_ERROR;
     }
 
   buflen = ntohl (len);
@@ -297,14 +374,14 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
   if (!buf)
     {
       *err_msg = "Not enough memory";
-      return 1;
+      return SHISHI_TOO_SMALL_BUFFER;
     }
 
   rc = read (infd, buf, buflen);
   if (rc != buflen)
     {
       *err_msg = "Error reading authentication message";
-      return 1;
+      return SHISHI_IO_ERROR;
     }
 
   len = strlen (krb5sendauth);
@@ -312,7 +389,9 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
   if (rc)
     {
       *err_msg = "Invalid authentication type";
-      return 1;
+      /* Authentication type is wrong.  */
+      write (infd, "\001", 1);
+      return SHISHI_VERIFY_FAILED;
     }
 
   free (buf);
@@ -322,21 +401,21 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
   if (rc != sizeof (int))
     {
       *err_msg = "Error reading protocol message size";
-      return 1;
+      return SHISHI_IO_ERROR;
     }
   buflen = ntohl (len);
   buf = malloc (buflen);
   if (!buf)
     {
       *err_msg = "Not enough memory";
-      return 1;
+      return SHISHI_TOO_SMALL_BUFFER;
     }
 
   rc = read (infd, buf, buflen);
   if (rc != buflen)
     {
       *err_msg = "Error reading protocol message";
-      return 1;
+      return SHISHI_IO_ERROR;
     }
 
   len = strlen (krb5kcmd1);
@@ -348,7 +427,9 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
       if (rc)
 	{
 	  *err_msg = "Protocol version not supported";
-	  return 1;
+	  /* Protocol version is wrong.  */
+	  write (infd, "\002", 1);
+	  return SHISHI_VERIFY_FAILED;
 	}
       *protoversion = 2;
     }
@@ -367,7 +448,7 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
   if (rc != sizeof (int))
     {
       *err_msg = "Error reading authentication request size";
-      return 1;
+      return SHISHI_IO_ERROR;
     }
 
   buflen = ntohl (len);
@@ -375,14 +456,14 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
   if (!buf)
     {
       *err_msg = "Not enough memory";
-      return 1;
+      return SHISHI_TOO_SMALL_BUFFER;
     }
 
   rc = read (infd, buf, buflen);
   if (rc != buflen)
     {
       *err_msg = "Error reading authentication request";
-      return 1;
+      return SHISHI_IO_ERROR;
     }
 
   /* Create Authentication context */
@@ -393,7 +474,7 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
 
   /* Store request in context */
 
-  shishi_ap_req_der_set (*ap, buf, buflen);
+  rc = shishi_ap_req_der_set (*ap, buf, buflen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -424,33 +505,36 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
   if (rc != SHISHI_OK)
     return rc;
 
-  /* User is authenticated. */
+  /* User is authenticated.  */
   error = 0;
   write (infd, &error, sizeof (int));
 
-  /* Authenticate ourself to client, if request */
+  /* Authenticate ourself to client, if requested.  */
 
   if (shishi_apreq_mutual_required_p (*handle, shishi_ap_req (*ap)))
     {
+      int len;
+
       rc = shishi_ap_rep_der (*ap, &out, &outlen);
       if (rc != SHISHI_OK)
 	return rc;
 
-      outlen = htonl (outlen);
-      rc = write (infd, &outlen, sizeof (int));
+      len = outlen;
+      len = htonl (len);
+      rc = write (infd, &len, sizeof (len));
       if (rc != sizeof (int))
 	{
 	  *err_msg = "Error sending AP-REP";
 	  free (out);
-	  return 1;
+	  return SHISHI_IO_ERROR;
 	}
 
-      rc = write (infd, out, ntohl (outlen));
-      if (rc != ntohl (outlen))
+      rc = write (infd, out, ntohl (len));
+      if (rc != (int) ntohl (len))
 	{
 	  *err_msg = "Error sending AP-REP";
 	  free (out);
-	  return 1;
+	  return SHISHI_IO_ERROR;
 	}
 
       free (out);
@@ -467,7 +551,7 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
       if (tkt == NULL)
 	{
 	  *err_msg = "Could not get tkt from AP-REQ";
-	  return 1;
+	  return SHISHI_INVALID_TICKET;
 	}
 
       rc = shishi_encticketpart_get_key (*handle,
@@ -485,15 +569,15 @@ get_auth (int infd, Shishi ** handle, Shishi_ap ** ap,
 
 /* read encrypted data on socket */
 int
-readenc (Shishi * h, int sock, char *buf, int *len, shishi_ivector * iv,
-	 Shishi_key * enckey, int proto)
+readenc (Shishi *h, int sock, char *buf, int *len, shishi_ivector *iv,
+	 Shishi_key *enckey, int proto)
 {
   char *out;
   char *outbis;
 
   int rc;
   int val;
-  int outlen;
+  size_t outlen;
   int dlen = 0, blocksize, enctype, hashsize;
 
   /* read size of message */
@@ -598,7 +682,7 @@ readenc (Shishi * h, int sock, char *buf, int *len, shishi_ivector * iv,
       val = sizeof (int);
     }
 
-  memset (buf, 0, BUFLEN);
+  memset (buf, 0, SHISHI_ENCRYPT_BUFLEN);
 
   /* copy decrypted data to output */
   memcpy (buf, out + val, outlen - val);
@@ -611,14 +695,15 @@ readenc (Shishi * h, int sock, char *buf, int *len, shishi_ivector * iv,
 
 /* write encrypted data to socket */
 int
-writeenc (Shishi * h, int sock, char *buf, int wlen, int *len,
-	  shishi_ivector * iv, Shishi_key * enckey, int proto)
+writeenc (Shishi *h, int sock, char *buf, int wlen, int *len,
+	  shishi_ivector *iv, Shishi_key *enckey, int proto)
 {
   char *out;
   char *bufbis;
 
   int rc;
-  int dlen, outlen;
+  int dlen;
+  size_t outlen;
 
   dlen = wlen;
   dlen = htonl (dlen);
@@ -677,5 +762,5 @@ writeenc (Shishi * h, int sock, char *buf, int wlen, int *len,
 
 
 }
-# endif	/* ENCRYPTION */
+# endif/* ENCRYPTION */
 #endif
