@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010, 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
+  Copyright (C) 2000-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -57,7 +56,7 @@ int revstatcmp (const FTSENT *, const FTSENT *);
 int sizecmp (const FTSENT *, const FTSENT *);
 int revsizecmp (const FTSENT *, const FTSENT *);
 
-char *flags_to_string (u_int, char *);
+char *flags_to_string (unsigned int, char *);
 int putname (char *);
 void printcol (DISPLAY *);
 void printacol (DISPLAY *);
