@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-  2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation,
-  Inc.
+  Copyright (C) 1997-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -57,11 +55,21 @@
 # include <pwd.h>
 # include <netdb.h>
 
-# include <krb5/krb5.h>
-# include <krb5/asn1.h>
-# include <krb5/crc-32.h>
-# include <krb5/los-proto.h>
-# include <krb5/ext-proto.h>
+# ifdef HAVE_KRB5_KRB5_H
+#  include <krb5/krb5.h>
+# endif
+# ifdef HAVE_KRB5_ASN1_H
+#  include <krb5/asn1.h>
+# endif
+# ifdef HAVE_KRB5_CRC_32_H
+#  include <krb5/crc-32.h>
+# endif
+# ifdef HAVE_KRB5_LOS_PROTO_H
+#  include <krb5/los-proto.h>
+# endif
+# ifdef HAVE_KRB5_EXT_PROTO_H
+#  include <krb5/ext-proto.h>
+# endif
 
 # define KRB5_DEFAULT_LIFE 60*60*8	/* 8 hours */
 /* helper function: convert flags to necessary KDC options */
@@ -334,8 +342,7 @@ mk_cred (dec_rep, etype, key, sender_addr, recv_addr, outbuf)
 
   krb5_use_cstype (&eblock, etype);
   ret_cred.enc_part.ciphertext.length = krb5_encrypt_size (scratch->length,
-							   eblock.
-							   crypto_entry);
+							   eblock.crypto_entry);
   /* add padding area, and zero it */
   if (!(scratch->data = realloc (scratch->data,
 				 ret_cred.enc_part.ciphertext.length)))
