@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1994-2023 Free Software Foundation, Inc.
+  Copyright (C) 1994-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -711,7 +711,7 @@ close_sep (struct servtab *sep)
     }
   sep->se_count = 0;
   /*
-   * Don't keep the pid of this running deamon: when reapchild()
+   * Don't keep the pid of this running daemon: when reapchild()
    * reaps this pid, it would erroneously increment nsock.
    */
   if (sep->se_wait > 1)
@@ -994,13 +994,13 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
   if (serv_node)
     return next_node_sep (sep);
 
-  memset ((caddr_t) sep, 0, sizeof *sep);
+  memset (sep, 0, sizeof *sep);
 
   while (1)
     {
       argcv_free (argc, argv);
       freeconfig (sep);
-      memset ((caddr_t) sep, 0, sizeof *sep);
+      memset (sep, 0, sizeof *sep);
 
       do
 	{
