@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1994-2024 Free Software Foundation, Inc.
+  Copyright (C) 1994-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -180,7 +180,7 @@
 # define DAY (24 * 60 * 60)
 #endif
 
-int keepalive = 1;		/* flag for SO_KEEPALIVE scoket option */
+int keepalive = 1;		/* flag for SO_KEEPALIVE socket option */
 int check_all;
 int log_success;		/* If TRUE, log all successful accesses */
 int reverse_required = 0;	/* Demand IP to host name resolution.  */
@@ -411,7 +411,7 @@ char username[32 + sizeof ("USER=")] = "USER=";
 char logname[32 + sizeof ("LOGNAME=")] = "LOGNAME=";
 char homedir[256 + sizeof ("HOME=")] = "HOME=";
 char shell[64 + sizeof ("SHELL=")] = "SHELL=";
-char path[sizeof (PATH_DEFPATH) + sizeof ("PATH=")] = "PATH=";
+char path[sizeof (PATH_DEFPATH) + sizeof ("PATH=") + 1] = "PATH=";
 char rhost[128 + sizeof ("RHOST=")] = "RHOST=";
 
 #ifndef WITH_PAM
@@ -524,7 +524,7 @@ doit (int sockfd, struct sockaddr *fromp, socklen_t fromlen)
   snprintf (portstr, sizeof (portstr), "%u", inport);
 #endif
 
-  /* Verify that the client's address is an Internet adress. */
+  /* Verify that the client's address is an Internet address. */
 #ifdef KERBEROS
   if (fromp->sa_family != AF_INET)
     {
