@@ -1,8 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
-  2015, 2016, 2017, 2018, 2019, 2020, 2021 Free Software Foundation,
-  Inc.
+  Copyright (C) 1995-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -112,7 +109,8 @@ stilloob (void)
   if (value < 0)
     {
       perror ("select");
-      (void) quit ();
+      (void) quit (0, NULL);
+      /* NOTREACHED */
     }
   if (FD_ISSET (net, &excepts))
     {
@@ -151,7 +149,7 @@ setneturg (void)
 int
 netflush (void)
 {
-  register int n, n1;
+  int n, n1;
 
 #ifdef	ENCRYPTION
   if (encrypt_output)
