@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
-  2011, 2012 Free Software Foundation, Inc.
+  Copyright (C) 2002-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -21,6 +20,8 @@
 
 #include <telnetd.h>
 
+#include <fcntl.h>
+
 typedef struct termios TERMDESC;
 #define _term_getattr tcgetattr
 #define _term_setattr(fd, tp) tcsetattr (fd, TCSANOW, tp)
@@ -30,7 +31,7 @@ TERMDESC termbuf, termbuf2;
 #ifdef IOCTL_INTERFACE
 
 void
-_term_setattr (int fd, TERMDESC * tp)
+_term_setattr (int fd, TERMDESC *tp)
 {
   ioctl (fd, TIOCSETN, (char *) &tp->sg);
   ioctl (fd, TIOCSETC, (char *) &tp->tc);
@@ -39,19 +40,19 @@ _term_setattr (int fd, TERMDESC * tp)
 }
 
 void
-term_send_eof ()
+term_send_eof (void)
 {
   /*nothing */
 }
 
 int
-term_change_eof ()
+term_change_eof (void)
 {
   return 0;
 }
 
 int
-tty_linemode ()
+tty_linemode (void)
 {
   return termbuf.state & TS_EXTPROC;
 }
@@ -63,30 +64,30 @@ tty_setlinemode (int on)
   set_termbuf ();
   ioctl (pty, TIOCEXT, (char *) &on);
   init_termbuf ();
-# else /* !TIOCEXT */
+# else/* !TIOCEXT */
 #  ifdef	EXTPROC
   if (on)
     termbuf.c_lflag |= EXTPROC;
   else
     termbuf.c_lflag &= ~EXTPROC;
 #  endif
-# endif	/* TIOCEXT */
+# endif/* TIOCEXT */
 }
 
 int
-tty_isecho ()
+tty_isecho (void)
 {
   return termbuf.sg.sg_flags & ECHO;
 }
 
 int
-tty_flowmode ()
+tty_flowmode (void)
 {
   return ((termbuf.tc.t_startc) > 0 && (termbuf.tc.t_stopc) > 0) ? 1 : 0;
 }
 
 int
-tty_restartany ()
+tty_restartany (void)
 {
 # ifdef	DECCTQ
   return (termbuf.lflags & DECCTQ) ? 0 : 1;
@@ -105,7 +106,7 @@ tty_setecho (int on)
 }
 
 int
-tty_israw ()
+tty_israw (void)
 {
   return termbuf.sg.sg_flags & RAW;
 }
@@ -140,25 +141,25 @@ tty_binaryout (int on)
 }
 
 int
-tty_isbinaryin ()
+tty_isbinaryin (void)
 {
   return termbuf.lflags & LPASS8;
 }
 
 int
-tty_isbinaryout ()
+tty_isbinaryout (void)
 {
   return termbuf.lflags & LLITOUT;
 }
 
 int
-tty_isediting ()
+tty_isediting (void)
 {
   return !(termbuf.sg.sg_flags & (CBREAK | RAW));
 }
 
 int
-tty_istrapsig ()
+tty_istrapsig (void)
 {
   return !(termbuf.sg.sg_flags & RAW);
 }
@@ -178,7 +179,7 @@ tty_setsig (int on)
 }
 
 int
-tty_issofttab ()
+tty_issofttab (void)
 {
   return termbuf.sg.sg_flags & XTABS;
 }
@@ -193,7 +194,7 @@ tty_setsofttab (int on)
 }
 
 int
-tty_islitecho ()
+tty_islitecho (void)
 {
   return !(termbuf.lflags & LCTLECH);
 }
@@ -208,12 +209,12 @@ tty_setlitecho (int on)
 }
 
 int
-tty_iscrnl ()
+tty_iscrnl (void)
 {
   return termbuf.sg.sg_flags & CRMOD;
 }
 
-#else
+#else /* !IOCTL_INTERFACE */
 
 # define termdesc_eofc    c_cc[VEOF]
 # define termdesc_erase   c_cc[VERASE]
@@ -289,7 +290,7 @@ tty_linemode (void)
   return (termbuf.c_lflag & EXTPROC);
 # else
   return 0;			/* Can't ever set it either. */
-# endif	/* EXTPROC */
+# endif/* EXTPROC */
 }
 
 void
@@ -299,14 +300,17 @@ tty_setlinemode (int on)
   set_termbuf ();
   ioctl (pty, TIOCEXT, (char *) &on);
   init_termbuf ();
-# else /* !TIOCEXT */
+# else/* !TIOCEXT */
 #  ifdef	EXTPROC
   if (on)
     termbuf.c_lflag |= EXTPROC;
   else
     termbuf.c_lflag &= ~EXTPROC;
+#  else
+  /* !EXTPROC */
+  (void) on;			/* Silence warnings.  */
 #  endif
-# endif	/* TIOCEXT */
+# endif/* TIOCEXT */
 }
 
 int
@@ -494,21 +498,45 @@ tty_iscrnl (void)
   return termbuf.c_iflag & ICRNL;
 }
 
-#endif
+#endif /* !IOCTL_INTERFACE */
 
 void
 init_termbuf (void)
 {
+#if !defined SOLARIS10 && !defined SOLARIS
   _term_getattr (pty, &termbuf);
+#else /* SOLARIS || SOLARIS10 */
+  /* On Solaris the master PTY is not able to report terminal
+   * settings about the slave TTY, since it is only the slave
+   * that is working with an designated line discipline.
+   * Therefore we must determine the slave descriptor, which
+   * exists only if ptsname() returns a non-empty string.
+   * This happens for the parent process.  The child process
+   * sees an empty name, so undergoes minimal processing.
+   */
+  int tty = pty;
+  char *name = ptsname (pty);
+
+  if (name)
+    /* Minimal access means read only!  */
+    tty = open (name, O_RDONLY | O_NONBLOCK);
+
+  _term_getattr (tty, &termbuf);
+
+  if (name)
+    close (tty);
+#endif /* SOLARIS || SOLARIS10 */
+
   termbuf2 = termbuf;
 }
 
 #if defined TIOCPKT_IOCTL
-/*FIXME: Hardly needed*/
+/*FIXME: Hardly needed?
+ * Built by OpenSolaris and BSD, though.  */
 void
-copy_termbuf ()
+copy_termbuf (void)
 {
-  int len = 0;
+  size_t len = 0;
   char *cp = (char *) &termbuf;
 
   while (pty_input_level () > 0)
@@ -516,6 +544,7 @@ copy_termbuf ()
       if (len >= sizeof (termbuf))
 	break;
       *cp++ = pty_get_char (0);
+      len++;
     }
   termbuf2 = termbuf;
 }
@@ -525,7 +554,23 @@ void
 set_termbuf (void)
 {
   if (memcmp (&termbuf, &termbuf2, sizeof (termbuf)))
+#if !defined SOLARIS10 && !defined SOLARIS
     _term_setattr (pty, &termbuf);
+#else /* SOLARIS || SOLARIS10 */
+    {
+      /* Same reason as with _term_getattr.  */
+      int tty = pty;
+      char *name = ptsname (pty);
+
+      if (name)
+	tty = open (name, O_RDWR | O_NONBLOCK | O_NOCTTY);
+
+      _term_setattr (tty, &termbuf);
+
+      if (name)
+	close (tty);
+    }
+#endif /* SOLARIS || SOLARIS10 */
 }
 
 /* spcset(func, valp, valpp)
@@ -545,7 +590,7 @@ set_termbuf (void)
 			return SLC_DEFAULT;
 
 int
-spcset (int func, cc_t * valp, cc_t ** valpp)
+spcset (int func, cc_t *valp, cc_t **valpp)
 {
   switch (func)
     {
@@ -655,48 +700,47 @@ struct termspeeds
 {
   int speed;
   int value;
-} termspeeds[] =
-  {
-    {0, B0},
-    {50, B50},
-    {75, B75},
-    {110, B110},
-    {134, B134},
-    {150, B150},
-    {200, B200},
-    {300, B300},
-    {600, B600},
-    {1200, B1200},
-    {1800, B1800},
-    {2400, B2400},
-    {4800, B4800},
+} termspeeds[] = {
+  {0, B0},
+  {50, B50},
+  {75, B75},
+  {110, B110},
+  {134, B134},
+  {150, B150},
+  {200, B200},
+  {300, B300},
+  {600, B600},
+  {1200, B1200},
+  {1800, B1800},
+  {2400, B2400},
+  {4800, B4800},
 # ifdef	B7200
-    {7200, B7200},
+  {7200, B7200},
 # endif
-    {9600, B9600},
+  {9600, B9600},
 # ifdef	B14400
-    {14400, B14400},
+  {14400, B14400},
 # endif
 # ifdef	B19200
-    {19200, B19200},
+  {19200, B19200},
 # endif
 # ifdef	B28800
-    {28800, B28800},
+  {28800, B28800},
 # endif
 # ifdef	B38400
-    {38400, B38400},
+  {38400, B38400},
 # endif
 # ifdef	B57600
-    {57600, B57600},
+  {57600, B57600},
 # endif
 # ifdef	B115200
-    {115200, B115200},
+  {115200, B115200},
 # endif
 # ifdef	B230400
-    {230400, B230400},
+  {230400, B230400},
 # endif
-    {-1, 0}
-  };
+  {-1, 0}
+};
 #endif /* DECODE_BAUD */
 
 void
