@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free
-  Software Foundation, Inc.
+  Copyright (C) 1993-2025 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -41,11 +39,13 @@
 # include <termcap.h>
 #elif defined HAVE_CURSES_TGETENT
 # include <curses.h>
-# include <term.h>
+# ifndef _XOPEN_CURSES
+#  include <term.h>
+# endif
 #endif
 
-#if defined HAVE_STREAMSPTY && defined HAVE_GETMSG \
-	&& defined HAVE_STROPTS_H
+#if defined HAVE_STREAMSPTY && defined HAVE_GETMSG	\
+  && defined HAVE_STROPTS_H
 # include <stropts.h>
 #endif
 
@@ -63,7 +63,7 @@ static int ncc;
 static char ptyibuf[BUFSIZ], *ptyip;
 static int pcc;
 
-int not42;
+extern int not42;
 
 static int
 readstream (int p, char *ibuf, int bufsize)
@@ -74,9 +74,9 @@ readstream (int p, char *ibuf, int bufsize)
   int flags = 0;
   int ret = 0;
   struct termios *tsp;
-#ifdef HAVE_TERMIO_H
+# ifdef HAVE_TERMIO_H
   struct termio *tp;
-#endif
+# endif
   struct iocblk *ip;
   char vstop, vstart;
   int ixon;
@@ -134,7 +134,7 @@ readstream (int p, char *ibuf, int bufsize)
 	  ixon = tsp->c_iflag & IXON;
 	  break;
 
-#ifdef HAVE_TERMIO_H
+# ifdef HAVE_TERMIO_H
 	case TCSETA:
 	case TCSETAW:
 	case TCSETAF:
@@ -143,7 +143,7 @@ readstream (int p, char *ibuf, int bufsize)
 	  vstart = tp->c_cc[VSTART];
 	  ixon = tp->c_iflag & IXON;
 	  break;
-#endif /* HAVE_TERMIO_H */
+# endif/* HAVE_TERMIO_H */
 
 	default:
 	  errno = EAGAIN;
@@ -274,6 +274,7 @@ net_read (void)
     {
       syslog (LOG_INFO, "telnetd:  peer died");
       cleanup (0);
+      /* NOT REACHED */
     }
   else if (ncc > 0)
     {
@@ -302,7 +303,7 @@ pty_output_byte (int c)
 void
 pty_output_datalen (const void *data, size_t len)
 {
-  if ((&ptyobuf[BUFSIZ] - pfrontp) > len)
+  if ((size_t) (&ptyobuf[BUFSIZ] - pfrontp) > len)
     ptyflush ();
   memcpy (pfrontp, data, len);
   pfrontp += len;
@@ -338,6 +339,7 @@ ptyflush (void)
       if (errno == EWOULDBLOCK || errno == EINTR)
 	return;
       cleanup (0);
+      /* NOT REACHED */
     }
 
   pbackp += n;
@@ -362,7 +364,7 @@ pty_get_char (int peek)
 int
 pty_input_putback (const char *str, size_t len)
 {
-  if (len > &ptyibuf[BUFSIZ] - ptyip)
+  if (len > (size_t) (&ptyibuf[BUFSIZ] - ptyip))
     len = &ptyibuf[BUFSIZ] - ptyip;
   strncpy (ptyip, str, len);
   pcc += len;
@@ -370,6 +372,14 @@ pty_input_putback (const char *str, size_t len)
   return 0;
 }
 
+/* pty_read()
+ *
+ * Read errors EWOULDBLOCK, EAGAIN, and EIO are
+ * tweaked into reporting zero bytes input.
+ * In particular, EIO is known to appear when
+ * reading off the master side, before having
+ * an active slave side.
+ */
 int
 pty_read (void)
 {
@@ -382,8 +392,9 @@ pty_read (void)
     pcc = 0;
   ptyip = ptyibuf;
 
-  DEBUG (debug_report, 1, debug_output_data ("ptyread %d chars\r\n", pcc));
-  DEBUG (debug_pty_data, 1, printdata ("pty", ptyip, pcc));
+  DEBUG (debug_report, 1,
+	 debug_output_data ("td: ptyread %d chars\r\n", pcc));
+  DEBUG (debug_pty_data, 1, printdata ("pd", ptyip, pcc));
   return pcc;
 }
 
@@ -402,19 +413,23 @@ pty_read (void)
 void
 io_drain (void)
 {
+  fd_set rfds;
+
   DEBUG (debug_report, 1, debug_output_data ("td: ttloop\r\n"));
   if (nfrontp - nbackp > 0)
     netflush ();
 
-again:
+  FD_ZERO (&rfds);
+  FD_SET (net, &rfds);
+  if (1 != select (net + 1, &rfds, NULL, NULL, NULL))
+    {
+      syslog (LOG_INFO, "ttloop:  select: %m\n");
+      exit (EXIT_FAILURE);
+    }
+
   ncc = read (net, netibuf, sizeof netibuf);
   if (ncc < 0)
     {
-      if (errno == EAGAIN)
-	{
-	  syslog (LOG_INFO, "ttloop: retrying");
-	  goto again;
-	}
       syslog (LOG_INFO, "ttloop:  read: %m\n");
       exit (EXIT_FAILURE);
     }
@@ -441,7 +456,7 @@ again:
 int
 stilloob (int s)
 {
-  static struct timeval timeout = { 0 };
+  static struct timeval timeout = { 0, 0 };
   fd_set excepts;
   int value;
 
@@ -470,10 +485,14 @@ stilloob (int s)
  * character.
  */
 char *
-nextitem (char *current)
+nextitem (char *current, const char *endp)
 {
+  if (current >= endp)
+    return NULL;
   if ((*current & 0xff) != IAC)
     return current + 1;
+  if (current + 1 >= endp)
+    return NULL;
 
   switch (*(current + 1) & 0xff)
     {
@@ -481,19 +500,21 @@ nextitem (char *current)
     case DONT:
     case WILL:
     case WONT:
-      return current + 3;
+      return current + 3 <= endp ? current + 3 : NULL;
 
     case SB:			/* loop forever looking for the SE */
       {
-	register char *look = current + 2;
+	char *look = current + 2;
 
-	for (;;)
-	  if ((*look++ & 0xff) == IAC && (*look++ & 0xff) == SE)
+	while (look < endp)
+	  if ((*look++ & 0xff) == IAC && look < endp
+	      && (*look++ & 0xff) == SE)
 	    return look;
 
-    default:
-	return current + 2;
+	return NULL;
       }
+    default:
+      return current + 2 <= endp ? current + 2 : NULL;
     }
 }				/* end of nextitem */
 
@@ -514,15 +535,16 @@ nextitem (char *current)
  * caller should be setting the urgent data pointer AFTER calling
  * us in any case.
  */
-#define wewant(p) \
-  ((nfrontp > p) && ((*p&0xff) == IAC) && \
-   ((*(p+1)&0xff) != EC) && ((*(p+1)&0xff) != EL))
+#define wewant(p)					\
+  ((nfrontp > p) && ((*p & 0xff) == IAC) &&		\
+   (nfrontp > p + 1 && (((*(p + 1) & 0xff) != EC) &&	\
+                        ((*(p + 1) & 0xff) != EL))))
 
 
 void
 netclear (void)
 {
-  register char *thisitem, *next;
+  char *thisitem, *next;
   char *good;
 
 #ifdef	ENCRYPTION
@@ -531,7 +553,7 @@ netclear (void)
   thisitem = netobuf;
 #endif /* ENCRYPTION */
 
-  while ((next = nextitem (thisitem)) <= nbackp)
+  while ((next = nextitem (thisitem, nbackp)) != NULL && next <= nbackp)
     thisitem = next;
 
   /* Now, thisitem is first before/at boundary. */
@@ -542,15 +564,18 @@ netclear (void)
   good = netobuf;		/* where the good bytes go */
 #endif /* ENCRYPTION */
 
-  while (nfrontp > thisitem)
+  while (thisitem != NULL && nfrontp > thisitem)
     {
       if (wewant (thisitem))
 	{
 	  int length;
 
-	  for (next = thisitem; wewant (next) && nfrontp > next;
-	       next = nextitem (next))
+	  for (next = thisitem;
+	       next != NULL && wewant (next) && nfrontp > next;
+	       next = nextitem (next, nfrontp))
 	    ;
+	  if (next == NULL)
+	    next = nfrontp;
 
 	  length = next - thisitem;
 	  memmove (good, thisitem, length);
@@ -559,7 +584,7 @@ netclear (void)
 	}
       else
 	{
-	  thisitem = nextitem (thisitem);
+	  thisitem = nextitem (thisitem, nfrontp);
 	}
     }
 
@@ -611,6 +636,7 @@ netflush (void)
       if (errno == EWOULDBLOCK || errno == EINTR)
 	return;
       cleanup (0);
+      /* NOT REACHED */
     }
 
   nbackp += n;
@@ -684,11 +710,11 @@ _gettermname (void)
   ttloop (sequenceIs (ttypesubopt, baseline));
 }
 
-/* FIXME: should be getterminaltype (char *user_name, size_t size)
-   Changes terminaltype.
+/*
+ * Changes terminaltype.
  */
 int
-getterminaltype (char *user_name)
+getterminaltype (char *uname, size_t len)
 {
   int retval = -1;
 
@@ -696,12 +722,29 @@ getterminaltype (char *user_name)
 #if defined AUTHENTICATION
   /*
    * Handle the Authentication option before we do anything else.
+   * Distinguish the available modes by level:
+   *
+   *   off:                     Authentication is forbidden.
+   *   none:                    Voluntary authentication.
+   *   user, valid, other:      Mandatory authentication only.
    */
-  send_do (TELOPT_AUTHENTICATION, 1);
-  ttloop (his_will_wont_is_changing (TELOPT_AUTHENTICATION));
+  if (auth_level < 0)
+    send_wont (TELOPT_AUTHENTICATION, 1);
+  else
+    {
+      if (auth_level > 0)
+	send_do (TELOPT_AUTHENTICATION, 1);
+      else
+	send_will (TELOPT_AUTHENTICATION, 1);
 
-  if (his_state_is_will (TELOPT_AUTHENTICATION))
-    retval = auth_wait (user_name);
+      ttloop (his_will_wont_is_changing (TELOPT_AUTHENTICATION));
+
+      if (his_state_is_will (TELOPT_AUTHENTICATION))
+	retval = auth_wait (uname, len);
+    }
+#else /* !AUTHENTICATION */
+  (void) uname;			/* Silence warning.  */
+  (void) len;			/* Silence warning.  */
 #endif
 
 #ifdef	ENCRYPTION
@@ -824,18 +867,24 @@ getterminaltype (char *user_name)
   return retval;
 }
 
+/*
+ * Exit status:
+ *
+ *  1   Accepted terminal type, or inconclusive,
+ *  0   Explicitly unsupported type.
+ */
 int
 terminaltypeok (char *s)
 {
-  char buf[1024];
+#ifdef HAVE_TGETENT
+  char buf[2048];
 
   if (terminaltype == NULL)
     return 1;
 
-#ifdef HAVE_TGETENT
   if (tgetent (buf, s) == 0)
-#endif
     return 0;
+#endif /* HAVE_TGETENT */
 
   return 1;
 }
@@ -892,7 +941,7 @@ debug_output_data (const char *fmt, ...)
  * Print telnet options and commands in plain text, if possible.
  */
 void
-printoption (register char *fmt, register int option)
+printoption (char *fmt, int option)
 {
   if (TELOPT_OK (option))
     debug_output_data ("%s %s\r\n", fmt, TELOPT (option));
@@ -908,19 +957,27 @@ printoption (register char *fmt, register int option)
 void
 printsub (int direction, unsigned char *pointer, int length)
 {
-  register int i;
+  int i = 0;
 
-#if defined AUTHENTICATION && defined ENCRYPTION
+#if defined AUTHENTICATION || defined ENCRYPTION
   char buf[512];
 #endif
 
+  /* Silence unwanted debugging to '/tmp/telnet.debug'.
+   *
+   * XXX: Better location?
+   */
+  if ((pointer[0] == TELOPT_AUTHENTICATION && debug_level[debug_auth] < 1)
+      || (pointer[0] == TELOPT_ENCRYPT && debug_level[debug_encr] < 1))
+    return;
+
   if (direction)
     {
       debug_output_data ("td: %s suboption ",
 			 direction == '<' ? "recv" : "send");
       if (length >= 3)
 	{
-	  register int j;
+	  int j;
 
 	  i = pointer[length - 2];
 	  j = pointer[length - 1];
@@ -1151,9 +1208,19 @@ printsub (int direction, unsigned char *pointer, int length)
 		debug_output_data ("(0x%x)", pointer[i + SLC_FLAGS]);
 	      debug_output_data (" %d;", pointer[i + SLC_VALUE]);
 
+	      /* Protocol enforced duplication of IAC depends on pre
+	       * and post modification of data, hence differs between
+	       * in, out, and no direction, i.e., recursive mode.
+	       * Some systems assign _POSIX_VDISABLE and IAC the same
+	       * value!  Heuristic experiments led to the following.
+	       * Recursive mode needs both steps as written here.
+	       */
 	      if ((pointer[i + SLC_VALUE] == IAC) &&
-		  (pointer[i + SLC_VALUE + 1] == IAC))
+		  (pointer[i + SLC_VALUE + 1] == IAC) && (direction != '<'))
 		i++;
+	      if ((pointer[i + SLC_VALUE] == IAC) &&
+		  (pointer[i + SLC_VALUE + 1] == IAC) && !direction)
+		i += 2;
 	    }
 
 	  for (; i < length; i++)
@@ -1168,14 +1235,15 @@ printsub (int direction, unsigned char *pointer, int length)
 	      break;
 	    }
 	  {
-	    char tbuf[32];
+	    char tbuf[sizeof ("|EDIT|TRAPSIG|SOFT_TAB|LIT_ECHO|ACK")];
+
 	    snprintf (tbuf, sizeof (tbuf), "%s%s%s%s%s",
 		      pointer[2] & MODE_EDIT ? "|EDIT" : "",
 		      pointer[2] & MODE_TRAPSIG ? "|TRAPSIG" : "",
 		      pointer[2] & MODE_SOFT_TAB ? "|SOFT_TAB" : "",
 		      pointer[2] & MODE_LIT_ECHO ? "|LIT_ECHO" : "",
 		      pointer[2] & MODE_ACK ? "|ACK" : "");
-	    debug_output_data ("%s", tbuf[1] ? &tbuf[1] : "0");
+	    debug_output_data ("%s", tbuf[0] ? &tbuf[1] : "0");
 	  }
 
 	  if (pointer[2] & ~(MODE_EDIT | MODE_TRAPSIG | MODE_ACK))
@@ -1194,8 +1262,8 @@ printsub (int direction, unsigned char *pointer, int length)
 
     case TELOPT_STATUS:
       {
-	register char *cp;
-	register int j, k;
+	char *cp;
+	int j, k;
 
 	debug_output_data ("STATUS");
 
@@ -1319,50 +1387,50 @@ printsub (int direction, unsigned char *pointer, int length)
 
 	env_common:
 	  {
-	    register int noquote = 2;
+	    char *quote = "";
 	    for (i = 2; i < length; i++)
 	      {
 		switch (pointer[i])
 		  {
 		  case NEW_ENV_VAR:
-		    debug_output_data ("\" VAR " + noquote);
-		    noquote = 2;
+		    debug_output_data ("%sVAR ", quote);
+		    quote = "";
 		    break;
 
 		  case NEW_ENV_VALUE:
-		    debug_output_data ("\" VALUE " + noquote);
-		    noquote = 2;
+		    debug_output_data ("%sVALUE ", quote);
+		    quote = "";
 		    break;
 
 		  case ENV_ESC:
-		    debug_output_data ("\" ESC " + noquote);
-		    noquote = 2;
+		    debug_output_data ("%sESC ", quote);
+		    quote = "";
 		    break;
 
 		  case ENV_USERVAR:
-		    debug_output_data ("\" USERVAR " + noquote);
-		    noquote = 2;
+		    debug_output_data ("%sUSERVAR ", quote);
+		    quote = "";
 		    break;
 
 		  default:
 		    if (isprint (pointer[i]) && pointer[i] != '"')
 		      {
-                        if (noquote)
+			if (strcmp (quote, "") == 0)
 			  {
 			    debug_output_data ("\"");
-			    noquote = 0;
+			    quote = "\" ";
 			  }
-			debug_output_datalen ((char*) &pointer[i], 1);
+			debug_output_datalen ((char *) &pointer[i], 1);
 		      }
 		    else
 		      {
-			debug_output_data ("\" %03o " + noquote, pointer[i]);
-			noquote = 2;
+			debug_output_data ("%s%03o ", quote, pointer[i]);
+			quote = "";
 		      }
 		    break;
 		  }
 	      }
-	    if (!noquote)
+	    if (strcmp (quote, "\" ") == 0)
 	      debug_output_data ("\"");
 	    break;
 	  }
@@ -1384,7 +1452,7 @@ printsub (int direction, unsigned char *pointer, int length)
 	case TELQUAL_IS:
 	  debug_output_data (" %s ", (pointer[1] == TELQUAL_IS) ?
 			     "IS" : "REPLY");
-	  if (AUTHTYPE_NAME_OK (pointer[2]))
+	  if (AUTHTYPE_NAME_OK (pointer[2]) && AUTHTYPE_NAME (pointer[2]))
 	    debug_output_data ("%s ", AUTHTYPE_NAME (pointer[2]));
 	  else
 	    debug_output_data ("%d ", pointer[2]);
@@ -1408,7 +1476,7 @@ printsub (int direction, unsigned char *pointer, int length)
 	  debug_output_data (" SEND ");
 	  while (i < length)
 	    {
-	      if (AUTHTYPE_NAME_OK (pointer[i]))
+	      if (AUTHTYPE_NAME_OK (pointer[i]) && AUTHTYPE_NAME (pointer[i]))
 		debug_output_data ("%s ", AUTHTYPE_NAME (pointer[i]));
 	      else
 		debug_output_data ("%d ", pointer[i]);
@@ -1429,7 +1497,7 @@ printsub (int direction, unsigned char *pointer, int length)
 	case TELQUAL_NAME:
 	  i = 2;
 	  debug_output_data (" NAME \"");
-	  debug_output_datalen (&pointer[i], length);
+	  debug_output_datalen ((char *) &pointer[i], length);
 	  i += length;
 	  debug_output_data ("\"");
 	  break;
@@ -1477,7 +1545,7 @@ printsub (int direction, unsigned char *pointer, int length)
 	      debug_output_data (" (partial suboption??\?)");
 	      break;
 	    }
-	  if (ENCTYPE_NAME_OK (pointer[2]))
+	  if (ENCTYPE_NAME_OK (pointer[2]) && ENCTYPE_NAME (pointer[2]))
 	    debug_output_data ("%s ", ENCTYPE_NAME (pointer[2]));
 	  else
 	    debug_output_data (" %d (unknown)", pointer[2]);
@@ -1491,7 +1559,7 @@ printsub (int direction, unsigned char *pointer, int length)
 	  debug_output_data (" SUPPORT ");
 	  while (i < length)
 	    {
-	      if (ENCTYPE_NAME_OK (pointer[i]))
+	      if (ENCTYPE_NAME_OK (pointer[i]) && ENCTYPE_NAME (pointer[i]))
 		debug_output_data ("%s ", ENCTYPE_NAME (pointer[i]));
 	      else
 		debug_output_data ("%d ", pointer[i]);
@@ -1521,21 +1589,26 @@ printsub (int direction, unsigned char *pointer, int length)
       if (TELOPT_OK (pointer[0]))
 	debug_output_data ("%s (unknown)", TELOPT (pointer[0]));
       else
-	debug_output_data ("%d (unknown)", pointer[i]);
+	debug_output_data ("%d (unknown)", pointer[0]);
       for (i = 1; i < length; i++)
 	debug_output_data (" %d", pointer[i]);
       break;
     }
-  debug_output_data ("\r\n");
+
+  /* Without direction, we are doing a recursive suboption printing.
+   * Suppress NL, which would otherwise misplace the SE marker.
+   */
+  if (direction)
+    debug_output_data ("\r\n");
 }
 
 /*
  * Dump a data buffer in hex and ascii to the output data stream.
  */
 void
-printdata (register char *tag, register char *ptr, register int cnt)
+printdata (char *tag, char *ptr, int cnt)
 {
-  register int i;
+  int i;
   char xbuf[30];
 
   while (cnt)
@@ -1544,8 +1617,8 @@ printdata (register char *tag, register char *ptr, register int cnt)
       debug_output_data ("%s: ", tag);
       for (i = 0; i < 20 && cnt; i++)
 	{
-	  debug_output_data ("%02x", *ptr);
-	  xbuf[i] = isprint (*ptr) ? *ptr : '.';
+	  debug_output_data ("%02x", (unsigned char) *ptr);
+	  xbuf[i] = isprint ((int) *ptr) ? *ptr : '.';
 
 	  if (i % 2)
 	    debug_output_data (" ");
@@ -1567,18 +1640,18 @@ net_write (unsigned char *str, int len)
 }
 
 void
-net_encrypt ()
+net_encrypt (void)
 {
 # ifdef	ENCRYPTION
   char *s = (nclearto > nbackp) ? nclearto : nbackp;
   if (s < nfrontp && encrypt_output)
     (*encrypt_output) ((unsigned char *) s, nfrontp - s);
   nclearto = nfrontp;
-# endif	/* ENCRYPTION */
+# endif/* ENCRYPTION */
 }
 
 int
-telnet_spin ()
+telnet_spin (void)
 {
   io_drain ();
   return 0;
@@ -1633,7 +1706,7 @@ _var_short_name (struct line_expander *exp)
     case 'd':
       time (&t);
       strftime (timebuf, sizeof (timebuf),
-		"%l:%M%P on %A, %d %B %Y", localtime (&t));
+		"%l:%M%p on %A, %d %B %Y", localtime (&t));
       return xstrdup (timebuf);
 
     case 'h':
@@ -1642,6 +1715,9 @@ _var_short_name (struct line_expander *exp)
     case 'l':
       return xstrdup (local_hostname);
 
+    case 'L':
+      return xstrdup (line);
+
     case 't':
       q = strchr (line + 1, '/');
       if (q)
@@ -1656,6 +1732,9 @@ _var_short_name (struct line_expander *exp)
     case 'u':
       return user_name ? xstrdup (user_name) : NULL;
 
+    case 'U':
+      return getenv ("USER") ? xstrdup (getenv ("USER")) : xstrdup ("");
+
     default:
       exp->state = EXP_STATE_ERROR;
       return NULL;
@@ -1668,6 +1747,8 @@ _var_short_name (struct line_expander *exp)
 char *
 _var_long_name (struct line_expander *exp, char *start, int length)
 {
+  (void) start;			/* Silence warnings until implemented.  */
+  (void) length;
   exp->state = EXP_STATE_ERROR;
   return NULL;
 }
